"use strict";

const fs = require("fs");
const DbService = require("moleculer-db");

/**
 * @typedef {import('moleculer').Context} Context Moleculer's Context
 */

module.exports = function (collection, config, populates) {
	const cacheCleanEventName = `cache.clean.${collection}`;

	const schema = {
		mixins: [DbService],

		actions: {
			// Disabling default actions
			list: false,
			find: false,
			count: false,
			create: false,
			insert: false,
			get: false,
			update: false,
			remove: false	
		},

		events: {
			/**
			 * Subscribe to the cache clean event. If it's triggered
			 * clean the cache entries for this service.
			 *
			 * @param {Context} ctx
			 */
			async [cacheCleanEventName]() {
				if (this.broker.cacher) {
					await this.broker.cacher.clean(`${this.fullName}.*`);
				}
			}
		},

		methods: {
			/**
			 * Send a cache clearing event when an entity changed.
			 *
			 * @param {String} type
			 * @param {any} json
			 * @param {Context} ctx
			 */
			async entityChanged(type, json, ctx) {
				ctx.broadcast(cacheCleanEventName);
			},

			/**
			 * Loading populate data to the collection.
			 * It is called in the DB.mixin after the database connection establishing & the collection is empty.
			 */
			async seedDB() {
				if (populates.length > 0)
					await this.adapter.insertMany(populates);
			}
		},

		async started() {
			// Check the count of items in the DB. If it's empty,
			// call the `seedDB` method of the service.
			if (this.seedDB) {
				const count = await this.adapter.count();
				if (count == 0) {
					this.logger.info(`The '${collection}' collection is empty. Seeding the collection...`);
					await this.seedDB();
					this.logger.info("Seeding is done. Number of records:", await this.adapter.count());
				}
			}
		}
	};

	if (config.database.use == "mongo") {
		// Mongo adapter
		const SpiritsMongoDbAdapter = require("spirits-npm/db-adapters/mongo-db-adapter");

		schema.adapter = new SpiritsMongoDbAdapter(config.database.implementations.mongo.uri);
		schema.collection = collection;
	} else if (config.database.use == "neDBMemory") {
		// NeDB memory adapter for testing
		schema.adapter = new DbService.MemoryAdapter();
	} else { // neDBFile
		// NeDB file DB adapter

		// Create data folder
		if (!fs.existsSync("./data")) {
			fs.mkdirSync("./data");
		}

		schema.adapter = new DbService.MemoryAdapter({ filename: `./data/${collection}.db` });
	}

	return schema;
};
